#!/usr/bin/perl -w
# File        : transdict.pl
# Author      : Nicola L C Talbot
# Description : Converts language/dictionary.txt file into
#               a \LaTeX\ file for use with the manual. (To make
#               it easier to update manual if menu names/items 
#               change.)

# transdict.pl <in file>
# read <in file>
# and write to dictionary.tex

unless ($#ARGV == 0)
{
   die "Syntax : $0 <dictionary file>\n";
}

$infile = $ARGV[0];

open INFD, $infile or die "Unable to open '$infile' $!\n";

open OUTFD, ">dictionary.tex"
   or die "Unable to open 'dictionary.tex' $!\n";

while (<INFD>)
{
   # remove comments
   s/#.*//;

   chomp;

   # skip blank lines
   next if m/^[\s]*$/;

   # skip entries that require arguments
   next if m/\$\d/;

   if (m/([^=]*)=(.*)/)
   {
      $key = $1;
      $val = $2;

      # skip the info.* messages or the messages containing html tags
      next if (($key=~/^info\./ and not $key eq 'info.text') or $val=~/<[\w\s]\+>/);

      $val=~s/\$(\d)/\\param{$1}/g;
      $val=~s/\\n/ /g;
      $val=~s/%/\\%/g;
      $val=~s/&amp;/\\\&/g;

      $val=~s/\.\.\./\\ensuremath{\\ldotp\\ldotp\\ldotp}\\spacefactor=1000\\relax/g;

      # trim trailing colons and spaces
      $val=~s/[: ]+$//g;

      print OUTFD "\\defappstring{$key}{$val}\n";
   }
   else
   {
      die "<key>=<value> pair expected on input line $. (found '$_')\n";
   }
}

close OUTFD;
close INFD;

1;
