/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.*;
import java.io.*;
import java.nio.file.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

public class CharCheckSelector extends JDialog
   implements ActionListener
{
   public CharCheckSelector(MakeJmlrBookGUI application)
   {
      super(application, application.getLabel("charcheckselector.title"), true);

      app = application;

      getContentPane().add(new JLabel(app.getLabel("charcheckselector.info")), "North");

      scrollPane = new JScrollPane();
      scrollPane.setBackground(Color.white);

      getContentPane().add(scrollPane, "Center");

      JPanel buttonPanel = new JPanel();
      getContentPane().add(buttonPanel, "South");

      buttonPanel.add(JMLRResources.createOkayButton(this));
      buttonPanel.add(JMLRResources.createCancelButton(this));

      JButton helpButton = new JButton(app.getLabel("button.help"));
      helpButton.setMnemonic(app.getMnemonic("button.help"));
      app.enableHelpOnButton(helpButton, "sec:charcheck");
      buttonPanel.add(helpButton);

      pack();
      setLocationRelativeTo(app);
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("cancel"))
      {
         setVisible(false);
      }
      else if (action.equals("okay"))
      {
         okay();
      }
   }


   public void okay()
   {
      success = true;
      setVisible(false);
   }

   public BookRoot requestFiles(BookRoot root)
      throws IOException
   {
      // make a copy of root that only contains
      // .tex and .bib files

      BookRoot root2 = root.copy
       (
          new java.io.FileFilter()
          {
             public boolean accept(File f)
             {
                if (f.isDirectory())
                {
                   return true;
                }

                String name = f.getName().toLowerCase();

                return name.endsWith(".tex") || name.endsWith(".bib");
             }
          }
       );

      tree = new CheckBoxTree(root2);

      scrollPane.setViewportView(tree);

      pack();
      setLocationRelativeTo(null);

      success = false;
      setVisible(true);

      return (success ? root2 : null);
   }

   private MakeJmlrBookGUI app;

   private JScrollPane scrollPane;

   private CheckBoxTree tree;

   private boolean success;
}

