/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import javax.swing.tree.*;

public class CheckBoxTree extends JTree implements MouseListener
{
   public CheckBoxTree(TreeNode root)
   {
      super(root);

      setCellRenderer(new CheckBoxTreeCellRenderer());
      setSelectionModel(new DefaultTreeSelectionModel());

      addMouseListener(this);

      getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0), "toggleselcheck");
      getActionMap().put("toggleselcheck", new AbstractAction()
      {
         public void actionPerformed(ActionEvent evt)
         {
            toggleCheckHighlighted();
         }
      });
   }

   public void mousePressed(MouseEvent evt)
   {
   }

   public void mouseReleased(MouseEvent evt)
   {
   }

   public void mouseExited(MouseEvent evt)
   {
   }

   public void mouseEntered(MouseEvent evt)
   {
   }

   public void mouseClicked(MouseEvent evt)
   {
      TreePath path = getPathForLocation(evt.getX(), evt.getY());

      if (InputEvent.getMaskForButton(MouseEvent.BUTTON1) == InputEvent.BUTTON1_DOWN_MASK
        && !evt.isShiftDown() && !evt.isControlDown()
        && path != null)
      {
         if (evt.getClickCount() == getToggleClickCount())
         {
            toggleClick = true;
         }
         else
         {
            Integer timerInterval = (Integer)
               Toolkit.getDefaultToolkit().getDesktopProperty(
                  "awt.multiClickInterval");

            javax.swing.Timer timer = new javax.swing.Timer(timerInterval.intValue(),
                new ToggleClickAction(path));

            timer.setRepeats(false);
            timer.start();
         }
      }
   }

   class ToggleClickAction implements ActionListener
   {
      private TreePath path;

      public ToggleClickAction(TreePath path)
      {
         this.path = path;
      }

      public void actionPerformed(ActionEvent evt)
      {
         if (toggleClick)
         {
            toggleClick = false;
         }
         else
         {
            toggleCheckNode(path);

            treeDidChange();
         }
      }
   }

   public void toggleCheckNode(TreePath path)
   {
      Object obj = path.getLastPathComponent();

      if (obj instanceof DefaultMutableTreeNode)
      {
         obj = ((DefaultMutableTreeNode)obj).getUserObject();
      }

      if (obj instanceof CheckBoxNode)
      {
         CheckBoxNode node = (CheckBoxNode)obj;

         node.setNodeSelected(!node.isNodeSelected(), true);
      }
   }

   public void setCheckNode(Object obj, boolean isChecked)
   {
      if (obj instanceof DefaultMutableTreeNode)
      {
         obj = ((DefaultMutableTreeNode)obj).getUserObject();
      }

      if (obj instanceof CheckBoxNode)
      {
         CheckBoxNode node = (CheckBoxNode)obj;

         node.setNodeSelected(isChecked, true);
      }
   }

   public void toggleCheckHighlighted()
   {
      TreePath[] highlightedPaths = getSelectionPaths();

      if (highlightedPaths == null)
      {
         return;
      }

      for (int i = 0; i < highlightedPaths.length; i++)
      {
         toggleCheckNode(highlightedPaths[i]);
      }

      treeDidChange();
   }

   public void expandAll()
   {
      int maxRows = getRowCount();

      if (maxRows < 1)
      {
         return;
      }

      for (int i = maxRows-1; i >= 0; i--)
      {
         expandRow(i);
      }
   }

   public void checkall()
   {
      Object root = getModel().getRoot();

      if (root == null)
      {
         return;
      }

      walk(root, new WalkListener()
      {
         public void walkAction(WalkEvent evt)
         {
            setCheckNode(evt.getNode(), true);
         }
      });

      treeDidChange();
   }

   public void checkallExceptMultiPagePdfs(MakeJmlrBookGUI app)
   {
      Object root = getModel().getRoot();

      if (root == null)
      {
         return;
      }

      walk(root, new MultiPageWalkListener(app));

      treeDidChange();
   }

   public void uncheckall()
   {
      Object root = getModel().getRoot();

      if (root == null)
      {
         return;
      }

      walk(root, new WalkListener()
      {
         public void walkAction(WalkEvent evt)
         {
            setCheckNode(evt.getNode(), false);
         }
      });

      treeDidChange();
   }

   protected boolean inNodeControl(TreePath path, int x, int y)
   {
      if (path == null || getModel().isLeaf(path.getLastPathComponent()))
      {
         return false;
      }

      int width = 18;

      Icon ic = UIManager.getIcon("Tree.expandedIcon");

      if (ic != null)
      {
         width = ic.getIconWidth();
      }

      Insets i = getInsets();

      int depth;

      if (isRootVisible())
      {
         depth = path.getPathCount()-1;
      }
      else
      {
         depth = path.getPathCount()-2;
      }

      double left = getRowBounds(getRowForPath(path)).getX() - width + i.left;

      return x >= left && x <= left+width;
   }

   public void walk(Object start, WalkListener listener)
   {
      TreeModel model = getModel();

      listener.walkAction(new WalkEvent(this, model,
         (start instanceof TreeNode ? ((TreeNode)start).getParent() : null),
         start, model.isLeaf(start)));

      for (int i = 0, n = model.getChildCount(start); i < n; i++)
      {
         Object child = model.getChild(start, i);

         boolean isLeaf = model.isLeaf(child);

         if (isLeaf)
         {
            listener.walkAction(new WalkEvent(this, model, start, child, isLeaf));
         }
         else
         {
            walk(child, listener);
         }
      }
   }

   private boolean toggleClick=false;
}

class CheckBoxTreeCellRenderer extends JPanel implements TreeCellRenderer
{
   public CheckBoxTreeCellRenderer()
   {
      super();
      checkBox = new JCheckBox("");
      checkBox.setOpaque(false);
      cellRenderer = new DefaultTreeCellRenderer();
      add(cellRenderer);
      add(checkBox);
      setOpaque(false);

      checkBox.setBorder(null);
   }

   public Component getTreeCellRendererComponent(JTree tree, Object val,
     boolean selected, boolean expanded, boolean leaf, int row,
     boolean hasFocus)
   {
      cellRenderer.getTreeCellRendererComponent(tree, "", selected,
         expanded, leaf, row, hasFocus);

      if (val instanceof DefaultMutableTreeNode)
      {
         val = ((DefaultMutableTreeNode)val).getUserObject();
      }

      if (val instanceof CheckBoxNode)
      {
         CheckBoxNode node = (CheckBoxNode)val;
         checkBox.setSelected(node.isNodeSelected());
         checkBox.setText(node.getNodeText());
         checkBox.setFont(cellRenderer.getFont());
      }

      highlighted = selected;

      return this;
   }

   public void paint(Graphics g)
   {
      Color bg;

      if (highlighted)
      {
         bg = cellRenderer.getBackgroundSelectionColor();
      }
      else
      {
         bg = cellRenderer.getBackgroundNonSelectionColor();

         if (bg == null)
         {
            bg = getBackground();
         }
      }

      if (bg != null)
      {
         g.setColor(bg);
         g.fillRect(0, 0, getWidth(), getHeight());
      }

      super.paint(g);
   }

   protected DefaultTreeCellRenderer cellRenderer;
   protected JCheckBox checkBox;
   protected boolean highlighted;
}

class MultiPageWalkListener implements WalkListener
{
   public MultiPageWalkListener(MakeJmlrBookGUI application)
   {
      app = application;
   }

   public void walkAction(WalkEvent evt)
   {
      Object obj = evt.getNode();

      if (obj instanceof DefaultMutableTreeNode)
      {
         obj = ((DefaultMutableTreeNode)obj).getUserObject();
      }

      if (obj instanceof FileElement)
      {
         FileElement node = ((FileElement)obj);

         boolean isChecked = true;

         if (node.getName().toLowerCase().endsWith(".pdf"))
         {
            File file = node.getFile();

            int pageCount = -1;

            try
            {
               pageCount = app.getPdfPageCount(file);
            }
            catch (Exception e)
            {
            }

            isChecked = (pageCount < 2);
         }

         node.setNodeSelected(isChecked, true);
      }

   }

   private MakeJmlrBookGUI app;
}
