/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class MenuButtonItem
{
   public MenuButtonItem(String menuItemLabel, char menuItemMnemonic,
      URL imageLocation, String actionCommand,
      KeyStroke keyStroke, ActionListener listener,
      String toolTip, String altText)
   {
      item = new JMenuItem(menuItemLabel);
      item.setMnemonic(menuItemMnemonic);
      item.setActionCommand(actionCommand);

      button = new JButton();
      button.setActionCommand(actionCommand);

      if (listener != null)
      {
         item.addActionListener(listener);
         button.addActionListener(listener);
      }

      if (keyStroke != null)
      {
         item.setAccelerator(keyStroke);
      }

      if (toolTip != null)
      {
         item.setToolTipText(toolTip);
         button.setToolTipText(toolTip);
      }

      if (imageLocation != null)
      {
         button.setIcon(new ImageIcon(imageLocation,
            altText == null ? menuItemLabel : altText));
      }
   }

   public void setText(String text)
   {
      item.setText(text);
   }

   public String getText()
   {
      return item.getText();
   }

   public void setEnabled(boolean enable)
   {
/*
      item.setEnabled(enable);
      button.setEnabled(enable);
*/
      if (enable)
      {
         enableboth();
      }
      else
      {
         disableboth();
      }
   }

   private void enableboth()
   {
      Thread thread = new Thread()
      {
         public void run()
         {
            item.setEnabled(true);
            button.setEnabled(true);
         }
      };
      thread.start();
      thread = null;
   }

   private void disableboth()
   {
      Thread thread = new Thread()
      {
         public void run()
         {
            item.setEnabled(false);
            button.setEnabled(false);
         }
      };
      thread.start();
      thread = null;
   }

   public boolean isEnabled() 
   {
      return item.isEnabled();
   }

   public JMenuItem getItem()
   {
      return item;
   }

   public JButton getButton()
   {
      return button;
   }

   private JMenuItem item;
   private JButton button;
}
