/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

public class PdfxProcessListener implements ProcessListener
{
   public PdfxProcessListener(MakeJmlrBookGUI application)
   {
      this.app = application;
      this.message = null;
      this.errors = null;
   }

   public void setProcess(Process process)
   {
      this.process = process;
   }

   public void processLine(int lineNum, String line)
   {
      app.setInfo(line);

      if (lineNum > 4 && !line.startsWith("Page"))
      {
         if (message == null)
         {
            message = line;
         }
         else if (!message.contains(line))
         {
            message += "\n"+line;
         }
      }
   }

   public void processErrorLine(int lineNum, String line)
   {
      if (errors == null)
      {
         errors = line;
      }
      else
      {
         errors += "\n" + line;
      }
   }

   public void error(Exception e)
   {
      app.error(e);
   }

   public String getMessage()
   {
      return message;
   }

   public String getErrors()
   {
      return errors;
   }

   public void terminateProcess()
   {
      if (thread != null)
      {
         MakeJmlrBookGUI.debug("Interrupting "+thread);
         thread.interrupt();
      }

      if (process != null)
      {
         MakeJmlrBookGUI.debug("Destroying "+process);
         process.destroy();
      }
   }

   public void setThread(Thread thread)
   {
      this.thread = thread;
   }

   public void setInterruptor(InterruptTimerTask interruptor)
   {
      this.interruptor = interruptor;
   }

   public InterruptTimerTask getInterruptor()
   {
      return interruptor;
   }

   private MakeJmlrBookGUI app;
   private String message;
   private String errors;
   private Process process;
   private Thread thread;
   private InterruptTimerTask interruptor;
}
