/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.io.*;
import java.net.MalformedURLException;
import java.nio.file.Path;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.regex.*;
import java.util.Calendar;
import java.text.Normalizer;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.latex.*;
import com.dickimawbooks.texparserlib.latex.jmlr.*;
import com.dickimawbooks.texparserlib.latex.inputenc.InputEncSty;
import com.dickimawbooks.texparserlib.latex2latex.*;
import com.dickimawbooks.texparserlib.bib.*;
import com.dickimawbooks.texparserlib.html.L2HStringConverter;

import com.dickimawbooks.makejmlrbookgui.*;

public class ImportPaperPanel extends JPanel
   implements ActionListener
{
   public ImportPaperPanel(NewBookDialog dialog, ArticlesPanel articlesPanel)
   {
      super(null);
      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
      setBorder(BorderFactory.createRaisedSoftBevelBorder());

      this.dialog = dialog;

      JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
      add(mainPanel);

      mainPanel.add(
         new JLabel(MakeJmlrBookGUI.getLabel("newbook.articles.import")));

      fileField = new FileField(this, dialog.getFileChooser());
      mainPanel.add(fileField);

      mainPanel.add(Box.createHorizontalStrut(20));

      upButton = JMLRResources.createUpButton(this);
      mainPanel.add(upButton);
      downButton = JMLRResources.createDownButton(this);
      mainPanel.add(downButton);

      mainPanel.add(JMLRResources.createImageButton("newbook.articles",
        "addarticle", "table/RowInsertAfter24.gif", this));

      mainPanel.add(Box.createHorizontalStrut(20));

      removeButton = JMLRResources.createRemoveButton(this);
      mainPanel.add(removeButton);

      reprintPanel = new JPanel();
      add(reprintPanel);

      reprintBox = new JCheckBox(
         MakeJmlrBookGUI.getLabel("newbook.articles.isreprint"), false);
      reprintBox.addActionListener(this);
      reprintBox.setActionCommand("isreprint");

      reprintPanel.add(reprintBox);

      pageRangeLabel = new JLabel(
         MakeJmlrBookGUI.getLabel("newbook.articles.reprintpages"));

      reprintPanel.add(pageRangeLabel);

      startPageField = new JTextField(3);
      reprintPanel.add(startPageField);

      pageRangeDashLabel = new JLabel("--");

      reprintPanel.add(pageRangeDashLabel);

      endPageField = new JTextField(3);
      reprintPanel.add(endPageField);

      enableReprintPages(false);

      setMaximumSize(getPreferredSize());

      reprintPanel.setVisible(dialog.isCiML());

      setTransferHandler(new ImportPaperTransferHandler(
        articlesPanel));

      addMouseListener(new MouseAdapter()
      {
         public void mousePressed(MouseEvent evt)
         {
            requestFocusInWindow();
            setSelected(true);
            TransferHandler handler = getTransferHandler();
            handler.exportAsDrag((JComponent)evt.getSource(), evt,
               TransferHandler.MOVE);
         }
      });
   }

   public String getTransferData()
   {
      ImportPapersPanel parent = (ImportPapersPanel)getParent();

      String name = parent.getName();

      if (name == null)
      {
         MakeJmlrBookGUI.debug("No name for ImportPapersPanel "+parent);
         throw new NullPointerException();
      }

      return String.format("%s:%d", name,
        parent.getComponentIndex(this));
   }

   public void setSelected(boolean select)
   {
      isSelected = select;
      repaint();
   }

   public Color getHighlight()
   {
      return highlight;
   }

   public Color getBackground()
   {
      return isSelected ? getHighlight() : super.getBackground();
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      ImportPapersPanel parent = (ImportPapersPanel)getParent();

      if (action.equals("remove"))
      {
         parent.remove(this);
         parent.updateButtons();
         parent.revalidate();
      }
      else if (action.equals("addarticle"))
      {
         parent.addRowAfter(this);
      }
      else if (action.equals("up"))
      {
         parent.moveRowUp(parent.getComponentIndex(this));
      }
      else if (action.equals("down"))
      {
         parent.moveRowDown(parent.getComponentIndex(this));
      }
      else if (action.equals("isreprint"))
      {
         boolean isReprint = reprintBox.isSelected();

         enableReprintPages(isReprint);

         if (isReprint)
         {
            startPageField.requestFocusInWindow();
         }
      }
   }

   public void enableRemoveButton(boolean enabled)
   {
      removeButton.setEnabled(enabled);
   }

   public void enableUpButton(boolean enabled)
   {
      upButton.setEnabled(enabled);
   }

   public void enableDownButton(boolean enabled)
   {
      downButton.setEnabled(enabled);
   }

   public void updatePanel()
   {
      reprintPanel.setVisible(dialog.isCiML());
      revalidate();
   }

   public void enableReprintPages(boolean enabled)
   {
      pageRangeLabel.setEnabled(enabled);
      pageRangeDashLabel.setEnabled(enabled);
      startPageField.setEnabled(enabled);
      endPageField.setEnabled(enabled);
   }

   public String getFileName()
   {
      return fileField.getFileName();
   }

   public File getFile()
   {
      return fileField.getFile();
   }

   public void setFile(File file)
   {
      fileField.setFile(file);
   }

   public boolean hasFile()
   {
      String filename = fileField.getFileName();

      return (filename != null && !filename.isEmpty());
   }

   public boolean hasFile(File file)
   {
      File f = fileField.getFile();

      if (f == null)
      {
         return false;
      }

      return f.equals(file);
   }

   public boolean isReprint()
   {
      return reprintPanel.isVisible() && reprintBox.isSelected();
   }

   public String getStartPage()
   {
      return startPageField.getText();
   }

   public String getEndPage()
   {
      return endPageField.getText();
   }

   public File getToImportFile()
   {
      return attributes == null ? null : attributes.getToImportFile();
   }

   public void setCimlImportFile(File base, BibEntry entry, TeXParser parser)
   {
      File[] fileList = base.listFiles(
         new NewBookDocumentFileFilter(entry.getId()));

      if (fileList.length == 0)
      {
         fileList = base.listFiles(
            new NewBookDocumentFileFilter(null, entry.getId(), null, true));

         if (fileList.length > 0)
         {
            fileList = fileList[0].listFiles(
               new NewBookDocumentFileFilter(entry.getId()));
         }
      }

      if (fileList.length > 0)
      {
         fileField.setFile(fileList[0]);
      }

      reprintBox.setSelected(true);
      enableReprintPages(true);

      BibValueList pageList = entry.getField("pages");

      String pages = null;

      if (pages == null)
      {
         try
         {
            pages = pageList.expand(parser).format();
         }
         catch (IOException e)
         {
            MakeJmlrBookGUI.debug(e);
         }
      }

      if (pages != null)
      {

         Pattern p = Pattern.compile(
          "\\s*(.*?)\\s*--?\\s*(.*?)\\s*");

         Matcher m = p.matcher(pages);

         if (m.matches())
         {
            startPageField.setText(m.group(1));
            endPageField.setText(m.group(2));
         }
         else
         {
            startPageField.setText(pages);
            endPageField.setText("");
         }
      }
   }

   public ImportFileAttributes getAttributes()
   {
      return attributes;
   }

   public void setAttributes(ImportFileAttributes at)
   {
      this.attributes = at;
   }

   public Charset getCharSet()
   {
      return attributes == null ? null : attributes.getCharset();
   }

   public Vector<LaTeXFile> getPackages()
   {
      return attributes == null ? null : attributes.getPackages();
   }

   public String toString()
   {
      return String.format("%s[file=%s,reprint=%s]", getClass().getSimpleName(),
        fileField.getFileName(), 
        reprintBox.isSelected() ? 
         "true("+startPageField.getText()+"--"+endPageField.getText()+")":
         "false");
   }

   private File impFile;
   private ImportFileAttributes attributes;

   private FileField fileField;

   private JCheckBox reprintBox;

   private JTextField startPageField, endPageField;

   private JLabel pageRangeLabel, pageRangeDashLabel;

   private JComponent reprintPanel;

   private JButton upButton, downButton, removeButton;

   private NewBookDialog dialog;

   private boolean isSelected = false;

   private Color highlight = Color.yellow;
}

