/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.io.File;
import com.dickimawbooks.texparserlib.TeXParser;

import com.dickimawbooks.makejmlrbookgui.*;

public class NewBookException extends Exception
{
   public NewBookException(String message)
   {
      this(-1, message);
   }

   public NewBookException(int lineNumber, String message)
   {
      this(null, lineNumber, message);
   }

   public NewBookException(TeXParser parser, String message)
   {
      this(parser.getCurrentFile(), parser.getLineNumber(), message);
   }

   public NewBookException(File file, int lineNumber, String message)
   {
      super(message);
      this.lineNumber = lineNumber;
      this.file = file;
   }

   public int getLineNumber()
   {
      return lineNumber;
   }

   public File getFile()
   {
      return file;
   }

   public String getDetails()
   {
      String message;

      if (lineNumber == -1)
      {
         message = getMessage();
      }
      else
      {
         message = MakeJmlrBookGUI.getLabelWithValues(
           "error.line", ""+lineNumber, getMessage());
      }

      if (file != null)
      {
         message = MakeJmlrBookGUI.getLabelWithValues(
           "error.file", file.getName(), message);
      }

      return message;
   }

   private int lineNumber = -1;
   private File file = null;
}
