/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.io.*;
import java.util.Calendar;
import java.util.Vector;
import java.util.Hashtable;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.dickimawbooks.texparserlib.bib.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class TitleInfoPanel extends JPanel
  implements ActionListener,NewBookPanel
{
   public TitleInfoPanel(NewBookDialog newBookDialog)
   {
      super(new BorderLayout());
      this.newBookDialog = newBookDialog;

      File defaultDir = new File(
         newBookDialog.getApplication().getProperties().getDefaultDirectory());

      JPanel bookTypePanel = new JPanel();

      add(bookTypePanel, BorderLayout.NORTH);

      bookTypePanel.add(new JLabel(MakeJmlrBookGUI.getLabel("newbook.type")));

      ButtonGroup bg = new ButtonGroup();

      jmlrButton = JMLRResources.createRadioButton("newbook", "jmlrwcp", 
        this, true);
      bg.add(jmlrButton);
      bookTypePanel.add(jmlrButton);

      cimlButton = JMLRResources.createRadioButton("newbook", "ciml", 
        this, false);
      bg.add(cimlButton);
      bookTypePanel.add(cimlButton);

      cardLayout = new CardLayout();

      cardPanel = new JPanel(cardLayout);
      add(cardPanel, BorderLayout.CENTER);

      Box jmlrComp = Box.createVerticalBox();
      cardLayout.addLayoutComponent(jmlrComp, "jmlrwcp");
      cardPanel.add(jmlrComp);

      JComponent[] labels = new JComponent[jmlrData.length];
      jmlrFields = new TitleField[jmlrData.length];

      int maxWidth = 0;

      for (int i = 0; i < jmlrData.length; i++)
      {
         Box box = Box.createHorizontalBox();
         jmlrComp.add(box);

         switch (jmlrData[i].getType())
         {
            case INT:
              jmlrFields[i] = new TitleIntField("newbook.jmlrwcp",
                 jmlrData[i].getTag());
            break;
            case TEXT:
              jmlrFields[i] = new TitleTextField("newbook.jmlrwcp",
                 jmlrData[i].getTag());
            break;
            case LIST:
              jmlrFields[i] = new TitleListField("newbook.jmlrwcp",
                 jmlrData[i].getTag(), (String)jmlrData[i].getArg());
            break;
            case MENU:
              jmlrFields[i] = new TitleMenuField("newbook.jmlrwcp",
                 jmlrData[i].getTag(), (String)jmlrData[i].getArg());
            break;
            case FILE:
              jmlrFields[i] = new TitleFileField("newbook.jmlrwcp",
                 jmlrData[i].getTag(), (String)jmlrData[i].getArg(), defaultDir);
            break;
            case SPINNER:
              jmlrFields[i] = new TitleSpinnerField("newbook.jmlrwcp",
                 jmlrData[i].getTag(), 
                 (TitleSpinnerFieldInfo)jmlrData[i].getArg());
            break;
            case CONTRIBUTORS:
              jmlrFields[i] = new TitleContributorsField("newbook.jmlrwcp",
                 jmlrData[i].getTag(), (String)jmlrData[i].getArg());
            break;
         }

         Object defValue = jmlrData[i].getDefaultValue();

         if (defValue != null)
         {
            jmlrFields[i].setFieldValue(defValue);
         }

         labels[i] = jmlrFields[i].getLabel();

         box.add(labels[i]);

         box.add(jmlrFields[i].getComponent());

         if (!jmlrData[i].isRequired())
         {
            box.add(new JLabel(MakeJmlrBookGUI.getLabel("newbook.optional")));
         }

         Dimension dim = labels[i].getPreferredSize();
         maxWidth = Math.max(maxWidth, dim.width);

         jmlrComp.add(Box.createVerticalStrut(2));
      }

      jmlrComp.add(Box.createVerticalGlue());

      for (int i = 0; i < labels.length; i++)
      {
         Dimension dim = labels[i].getPreferredSize();
         dim.width = maxWidth;
         labels[i].setPreferredSize(dim);
      }

      Box cimlComp = Box.createVerticalBox();
      cardLayout.addLayoutComponent(cimlComp, "ciml");
      cardPanel.add(cimlComp);

      JComponent comp = new JPanel();
      comp.add(JMLRResources.createActionButton("newbook", "ciml.importbib",
        this, null));
      cimlComp.add(comp);

      labels = new JComponent[cimlData.length];
      cimlFields = new TitleField[cimlData.length];

      maxWidth = 0;

      for (int i = 0; i < cimlData.length; i++)
      {
         Box box = Box.createHorizontalBox();
         cimlComp.add(box);

         switch (cimlData[i].getType())
         {
            case INT:
               cimlFields[i] = new TitleIntField("newbook.ciml",
                  cimlData[i].getTag());
            break;
            case TEXT:
               cimlFields[i] = new TitleTextField("newbook.ciml",
                  cimlData[i].getTag());
            break;
            case LIST:
               cimlFields[i] = new TitleListField("newbook.ciml",
                  cimlData[i].getTag(), (String)cimlData[i].getArg());
            break;
            case MENU:
               cimlFields[i] = new TitleMenuField("newbook.ciml",
                  cimlData[i].getTag(), (String)cimlData[i].getArg());
            break;
            case FILE:
               cimlFields[i] = new TitleFileField("newbook.ciml",
                  cimlData[i].getTag(), (String)cimlData[i].getArg(), defaultDir);
            break;
            case SPINNER:
               cimlFields[i] = new TitleSpinnerField("newbook.ciml",
                  cimlData[i].getTag(), 
                  (TitleSpinnerFieldInfo)cimlData[i].getArg());
            break;
            case CONTRIBUTORS:
               cimlFields[i] = new TitleContributorsField("newbook.ciml",
                  cimlData[i].getTag(), (String)cimlData[i].getArg());
            break;
         }

         Object defValue = jmlrData[i].getDefaultValue();

         if (defValue != null)
         {
            cimlFields[i].setFieldValue(defValue);
         }

         labels[i] = cimlFields[i].getLabel();

         box.add(labels[i]);

         box.add(cimlFields[i].getComponent());

         if (!cimlData[i].isRequired())
         {
            box.add(new JLabel(MakeJmlrBookGUI.getLabel("newbook.optional")));
         }

         Dimension dim = labels[i].getPreferredSize();
         maxWidth = Math.max(maxWidth, dim.width);

         cimlComp.add(Box.createVerticalStrut(2));
      }

      cimlComp.add(Box.createVerticalGlue());

      for (int i = 0; i < labels.length; i++)
      {
         Dimension dim = labels[i].getPreferredSize();
         dim.width = maxWidth;
         labels[i].setPreferredSize(dim);
      }
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("jmlrwcp"))
      {
         cardLayout.show(cardPanel, "jmlrwcp");
      }
      else if (action.equals("ciml"))
      {
         cardLayout.show(cardPanel, "ciml");
      }
      else if (action.equals("ciml.importbib"))
      {
         JFileChooser bibFileChooser = newBookDialog.getBibFileChooser();

         if (bibFileChooser.showOpenDialog(this)
           != JFileChooser.APPROVE_OPTION)
         {
            return;
         }

         try
         {
            parseBib(bibFileChooser.getSelectedFile());
         }
         catch (IOException e)
         {
            JMLRResources.error(this, e);
         }
      }
   }

   public void parseBib(File file) throws IOException
   {
      BookTeXApp texapp = new BookTeXApp(getApplication());

      BibParser bibParser = new BibParser(texapp);
      bibParser.parseBibFile(file);

      Vector<BibData> bibdata = bibParser.getBibData();

      newBookDialog.clearImportData();

      for (BibData data : bibdata)
      {
         if (data instanceof BibEntry)
         {
            BibEntry entry = (BibEntry)data;
            String entryType = entry.getEntryType().toLowerCase();

            Hashtable<String,String> sectionMap = new Hashtable<String,String>();

            if (entryType.equals("proceedings"))
            {
               Vector<Contributor> editors = entry.getEditors(bibParser.getParser());

               if (editors != null)
               {
                  cimlFields[CIML_EDITOR].setFieldValue(editors);
               }

               BibValueList valList = entry.getField("booktitle");

               if (valList != null)
               {
                  cimlFields[CIML_TITLE].setFieldValue(
                    valList.expand(bibParser.getParser()).format());
               }

               valList = entry.getField("volume");

               if (valList != null)
               {
                  cimlFields[CIML_WCP_VOLUME].setFieldValue(Integer.parseInt(
                     valList.expand(bibParser.getParser()).format().trim()));
               }

               valList = entry.getField("sections");

               if (valList != null)
               {
                  String sections = 
                    valList.expand(bibParser.getParser()).format().trim();

                  if (!sections.isEmpty())
                  {
                     String[] sectionSplit = sections.split(" *| *");

                     for (int i = 0; i < sectionSplit.length; i++)
                     {
                        String[] split = sectionSplit[i].split(" *= *", 2);

                        if (split.length == 2)
                        {
                           sectionMap.put(split[0], split[1]);
                        }

                        if (!split[0].equals("preface"))
                        {
                           newBookDialog.addPart(split[1]);
                        }
                     }
                  }
               }
            }
            else if (entryType.equals("inproceedings"))
            {
               BibValueList valList = entry.getField("section");

               String section = (valList == null ? null :
                  valList.expand(bibParser.getParser()).format());

               if (section != null && section.equals("preface"))
               {
                  File base = file.getParentFile();

                  File[] fileList = base.listFiles(
                   new NewBookDocumentFileFilter(section, entry.getId()));

                  if (fileList.length == 0)
                  {
                     File[] dirList = base.listFiles(new FileFilter()
                      {
                         public boolean accept(File file)
                         {
                            return file.isDirectory();
                         }
                      });

                     for (int i = 0; i < dirList.length; i++)
                     {
                        fileList = dirList[i].listFiles(
                           new NewBookDocumentFileFilter(section, entry.getId()));

                        if (fileList.length > 0) break;
                     }
                  }

                  if (fileList.length > 0)
                  {
                     File srcFile = null;

                     for (int i = 0; i < fileList.length; i++)
                     {
                        if (fileList[i].getName().toLowerCase().equals("preface.tex"))
                        {
                           srcFile = fileList[i];
                           break;
                        }
                     }

                     if (srcFile == null)
                     {
                        srcFile = hasPrefaceEnv(fileList);
                     }

                     if (srcFile != null)
                     {
                        newBookDialog.setPrefaceFile(srcFile);
                     }
                  }
               }
               else
               {
                  if (section != null)
                  {
                     section = sectionMap.get(section);
                  }

                  newBookDialog.addCimlImportFile(file.getParentFile(),
                    section, entry, bibParser.getParser());
               }

            }
         }
      }

      String errors = texapp.getErrors();
      String warnings = texapp.getWarnings();

      if (!errors.isEmpty())
      {
         getApplication().error(errors);
      }

      if (!warnings.isEmpty())
      {
         getApplication().warning(warnings);
      }
   }

   private File hasPrefaceEnv(File[] fileList)
   {
      BufferedReader in = null;

      try
      {
         for (int i = 0; i < fileList.length; i++)
         {
            in = new BufferedReader(new FileReader(fileList[i]));

            String line;

            while ((line = in.readLine()) != null)
            {
               if (line.contains("\\begin{preface}"))
               {
                  in.close();
                  in = null;
                  return fileList[i];
               }
            }

            in.close();
            in = null;
         }
      }
      catch (IOException e)
      {
         getApplication().debug(e);
      }
      finally
      {
         if (in != null)
         {
            try
            {
               in.close();
            }
            catch (IOException e)
            {
               getApplication().debug(e);
            }
         }
      }

      return null;
   }

   public MakeJmlrBookGUI getApplication()
   {
      return newBookDialog.getApplication();
   }

   public boolean canMoveForwards()
   {
      String message = "";

      if (jmlrButton.isSelected())
      {
         for (int i = 0; i < jmlrData.length; i++)
         {
            TitleField field = (TitleField)jmlrFields[i];

            if (jmlrData[i].isRequired() && field.isEmpty())
            {
               String fieldLabel = MakeJmlrBookGUI.getLabelWithAlt(
                    "newbook.jmlrwcp."+jmlrData[i].getTag(),
                    jmlrData[i].getTag());

               int idx = fieldLabel.lastIndexOf(":");

               if (idx > -1)
               {
                  fieldLabel = fieldLabel.substring(0, idx);
               }

               message += MakeJmlrBookGUI.getLabelWithValue(
                 "error.missing_field", fieldLabel)+"\n";
            }
         }
      }
      else
      {
         for (int i = 0; i < cimlData.length; i++)
         {
            TitleField field = (TitleField)cimlFields[i];

            if (cimlData[i].isRequired() && field.isEmpty())
            {
               String fieldLabel = MakeJmlrBookGUI.getLabelWithAlt(
                   "newbook.ciml."+cimlData[i].getTag(),
                   cimlData[i].getTag());

               int idx = fieldLabel.lastIndexOf(":");

               if (idx > -1)
               {
                  fieldLabel = fieldLabel.substring(0, idx);
               }

               message += MakeJmlrBookGUI.getLabelWithValue(
                 "error.missing_field", fieldLabel)+"\n";
            }
         }
      }

      if (message.isEmpty())
      {
         return true;
      }

      JMLRResources.error(this, message);

      return false;
   }

   public boolean isCiML()
   {
      return cimlButton.isSelected();
   }

   public int getCiMLVolume()
   {
      return cimlFields[CIML_VOLUME].getIntValue();
   }

   public int getCiMLWCPVolume()
   {
      return cimlFields[CIML_WCP_VOLUME].getIntValue();
   }

   public int getJMLRVolume()
   {
      return jmlrFields[JMLR_VOLUME].getIntValue();
   }

   public int getJMLRYear()
   {
      return jmlrFields[JMLR_YEAR].getIntValue();
   }

   public String getJMLRMonth()
   {
      return jmlrFields[JMLR_MONTH].getText();
   }

   public String getJMLRDay()
   {
      return jmlrFields[JMLR_DAY].getText();
   }

   public String getJMLRLocation()
   {
      return jmlrFields[JMLR_LOCATION].getText();
   }

   public Vector<Contributor> getCiMLEditors()
   {
      return cimlFields[CIML_EDITOR].getContributors();
   }

   public Vector<Contributor> getJMLREditors()
   {
      return jmlrFields[JMLR_EDITOR].getContributors();
   }

   public String getJMLRTitle()
   {
      return jmlrFields[JMLR_TITLE].getText();
   }

   public String getJMLRShortTitle()
   {
      return jmlrFields[JMLR_SHORT_TITLE].getText();
   }

   public String getCiMLTitle()
   {
      return cimlFields[CIML_TITLE].getText();
   }

   public String getCiMLShortTitle()
   {
      return cimlFields[CIML_SHORT_TITLE].getText();
   }

   public String getCiMLSubTitle()
   {
      return cimlFields[CIML_SUBTITLE].getText();
   }

   public String getCiMLProductionEditor()
   {
      return cimlFields[CIML_PRODEDITOR].getText();
   }

   public void updatePanel() {}

   public boolean hasNextButton() {return true;}

   public boolean hasBackButton() {return true;}

   private NewBookDialog newBookDialog;

   private CardLayout cardLayout;

   private JComponent cardPanel;

   private JRadioButton jmlrButton, cimlButton;

   private TitleField[] jmlrFields, cimlFields;

   private static final int TEXT=0, INT=1, LIST=2, MENU=3, FILE=4, SPINNER=5,
     CONTRIBUTORS=6;

   private static final int JMLR_VOLUME=0;
   private static final int JMLR_TITLE=1;
   private static final int JMLR_SHORT_TITLE=2;
   private static final int JMLR_YEAR=3;
   private static final int JMLR_MONTH=4;
   private static final int JMLR_DAY=5;
   private static final int JMLR_LOCATION=6;
   private static final int JMLR_EDITOR=7;

   private static final TitleInfoData[] jmlrData =
   {
      new TitleInfoData("volume", null, false, INT),
      new TitleInfoData("title", null, true, TEXT),
      new TitleInfoData("shorttitle", null, true, TEXT),
      new TitleInfoData("year", 
       new TitleIntSpinnerFieldInfo(
          Calendar.getInstance().get(Calendar.YEAR), 2000, 2100, 1, "####"
       ),
       true, SPINNER),
      new TitleInfoData("month", "newbook.months", true, MENU),
      new TitleInfoData("day", null, false, TEXT),
      new TitleInfoData("location", null, false, TEXT),
      new TitleInfoData("editor", "newbook.editor", true, CONTRIBUTORS)
   };

   private static final int CIML_VOLUME=0;
   private static final int CIML_WCP_VOLUME=1;
   private static final int CIML_TITLE=2;
   private static final int CIML_SHORT_TITLE=3;
   private static final int CIML_SUBTITLE=4;
   private static final int CIML_PRODEDITOR=5;
   private static final int CIML_EDITOR=6;

   private static final TitleInfoData[] cimlData =
   {
      new TitleInfoData("volume", null, true, INT),
      new TitleInfoData("wcpvolume", null, false, INT),
      new TitleInfoData("title", null, true, TEXT),
      new TitleInfoData("shorttitle", null, false, TEXT),
      new TitleInfoData("subtitle", null, false, TEXT),
      new TitleInfoData("prodeditor", null, false, TEXT),
      new TitleInfoData("editor", "newbook.editor", true, CONTRIBUTORS)
   };
}
